{-
   Copyright 2020 Morgan Stanley

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
-}


module Morphir.IR.SDK.Regex exposing (..)

import Dict
import Morphir.IR.Module as Module exposing (ModuleName)
import Morphir.IR.Name as Name
import Morphir.IR.Path as Path exposing (Path)
import Morphir.IR.Type as Type exposing (Specification(..), Type(..))
import Morphir.IR.Value as Value
import Morphir.IR.Documented exposing (Documented)


moduleName : ModuleName
moduleName =
    Path.fromString "Regex"


moduleSpec : Module.Specification ()
moduleSpec =
    { types =
         Dict.fromList
            [ ( Name.fromString "Regex", OpaqueTypeSpecification [] |> Documented "Type that represents regular expressions" )
            ]
    , values =
        let
            -- Used temporarily as a placeholder for function values until we can generate them based on the SDK.
            dummyValueSpec : Value.Specification ()
            dummyValueSpec =
                Value.Specification [] (Type.Unit ())

            valueNames : List String
            valueNames =
                [ "fromString"
                , "fromStringWith"
                , "never"
                , "contains"
                , "split"
                , "find"
                , "replace"
                , "splitAtMost"
                , "findAtMost"
                , "replaceAtMost"
                ]
        in
        valueNames
            |> List.map
                (\valueName ->
                    ( Name.fromString valueName, Documented "" dummyValueSpec )
                )
            |> Dict.fromList
    , doc = Just "Regular Expressions and related functions."
    }
